/*
 * This file is part of Generic Data Structures Library (GDSL).
 * Copyright (C) 1998-2004 Nicolas Darnis <ndarnis@free.fr>
 *
 * The GDSL library is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License as 
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The GDSL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the GDSL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * $RCSfile: main_llbintree.c,v $
 * $Revision: 1.11 $
 * $Date: 2004/08/08 12:00:21 $
 */

#include <config.h>
#include <stdio.h> 
#include <string.h>
#include <stdlib.h>


#include "_gdsl_bintree.h"
#include "_strings.h"


static int 
my_display_string (_gdsl_bintree_t t, void* d)
{
    print_string (_gdsl_bintree_get_content (t), stdout, d);
    return 1;
}

int main (void)
{
    _gdsl_bintree_t g, d, t, t1, t2, copy;

    g  = _gdsl_bintree_alloc ((gdsl_element_t) "b", NULL, NULL);
    d  = _gdsl_bintree_alloc ((gdsl_element_t) "f", NULL, NULL);
    t1 = _gdsl_bintree_alloc ((gdsl_element_t) "d", g, d);
    g  = _gdsl_bintree_alloc ((gdsl_element_t) "n", NULL, NULL);
    d  = _gdsl_bintree_alloc ((gdsl_element_t) "q", NULL, NULL);
    t2 = _gdsl_bintree_alloc ((gdsl_element_t) "o", g, d);
    t  = _gdsl_bintree_alloc ((gdsl_element_t) "k", t1, t2);
  
    printf ("T:\n");
    _gdsl_bintree_write_xml (t, print_string, stdout, NULL);

    copy = _gdsl_bintree_copy (t, copy_string);
    printf ("COPY OF T: \n");
    _gdsl_bintree_dump (copy, print_string, stdout, NULL);

    _gdsl_bintree_rotate_left (&t);
    _gdsl_bintree_rotate_right (&t);
    _gdsl_bintree_rotate_right (&t);
    _gdsl_bintree_rotate_left (&t);

    printf ("\nT in prefixed order: ");
    _gdsl_bintree_map_prefix (t, my_display_string, (void*) " ");
    printf ("\nT in infixed order: ");
    _gdsl_bintree_map_infix (t, my_display_string, (void*) " ");
    printf ("\nT in postfixed order: ");
    _gdsl_bintree_map_postfix (t, my_display_string, (void*) " ");

    printf ("\n\nCOPY OF T in prefixed order: ");
    _gdsl_bintree_map_prefix (copy, my_display_string, (void*) " ");
    printf ("\nCOPY OF T in infixed order: ");
    _gdsl_bintree_map_infix (copy, my_display_string, (void*) " ");
    printf ("\nCOPY OF T in postfixed order: ");
    _gdsl_bintree_map_postfix (copy, my_display_string, (void*) " ");
    printf ("\n\n");

    _gdsl_bintree_free (copy, free_string);
    _gdsl_bintree_free (t, NULL);

    exit (EXIT_SUCCESS);
}

/** EMACS **
 * Local variables:
 * mode: c
 * c-basic-offset: 4
 * End:
 */
